@@ -31,10 +31,6 @@
 #include <libpeas/peas-extension-set.h>
 #include <tepl/tepl.h>
 
-#ifdef ENABLE_INTROSPECTION
-#include <girepository.h>
-#endif
-
 #include "gedit-commands-private.h"
 #include "gedit-notebook.h"
 #include "gedit-debug.h"
@@ -57,12 +53,9 @@ typedef struct
 
 	GtkCssProvider     *theme_provider;
 
-	GeditLockdownMask  lockdown;
-
 	GtkPageSetup      *page_setup;
 	GtkPrintSettings  *print_settings;
 
-	GeditSettings     *settings;
 	GSettings         *ui_settings;
 	GSettings         *window_settings;
 
@@ -84,15 +77,6 @@ typedef struct
 	GApplicationCommandLine *command_line;
 } GeditAppPrivate;
 
-enum
-{
-	PROP_0,
-	PROP_LOCKDOWN,
-	LAST_PROP
-};
-
-static GParamSpec *properties[LAST_PROP];
-
 static const GOptionEntry options[] =
 {
 	/* Version */
@@ -152,7 +136,7 @@ static const GOptionEntry options[] =
 	{NULL}
 };
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GeditApp, gedit_app, GTK_TYPE_APPLICATION)
+G_DEFINE_TYPE_WITH_PRIVATE (GeditApp, gedit_app, GTK_TYPE_APPLICATION)
 
 static void
 gedit_app_dispose (GObject *object)
@@ -163,7 +147,6 @@ gedit_app_dispose (GObject *object)
 
 	g_clear_object (&priv->ui_settings);
 	g_clear_object (&priv->window_settings);
-	g_clear_object (&priv->settings);
 
 	g_clear_object (&priv->page_setup);
 	g_clear_object (&priv->print_settings);
@@ -190,25 +173,6 @@ gedit_app_dispose (GObject *object)
 	G_OBJECT_CLASS (gedit_app_parent_class)->dispose (object);
 }
 
-static void
-gedit_app_get_property (GObject    *object,
-			guint       prop_id,
-			GValue     *value,
-			GParamSpec *pspec)
-{
-	GeditApp *app = GEDIT_APP (object);
-
-	switch (prop_id)
-	{
-		case PROP_LOCKDOWN:
-			g_value_set_flags (value, gedit_app_get_lockdown (app));
-			break;
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-			break;
-	}
-}
-
 static gchar *
 gedit_app_help_link_id_impl (GeditApp    *app,
                              const gchar *name,
@@ -284,82 +248,19 @@ gedit_app_set_window_title_impl (GeditApp    *app,
 	gtk_window_set_title (GTK_WINDOW (window), title);
 }
 
-static gboolean
-is_in_viewport (GtkWindow    *window,
-		GdkScreen    *screen,
-		gint          workspace,
-		gint          viewport_x,
-		gint          viewport_y)
-{
-	GdkScreen *s;
-	GdkDisplay *display;
-	GdkMonitor *monitor;
-	GdkWindow *gdkwindow;
-	GdkRectangle geometry;
-	const gchar *cur_name;
-	const gchar *name;
-	gint ws;
-	gint x, y, width, height;
-
-	/* Check for screen and display match */
-	display = gdk_screen_get_display (screen);
-	cur_name = gdk_display_get_name (display);
-
-	s = gtk_window_get_screen (window);
-	display = gdk_screen_get_display (s);
-	name = gdk_display_get_name (display);
-
-	if (strcmp (cur_name, name) != 0)
-	{
-		return FALSE;
-	}
-
-	/* Check for workspace match */
-	ws = gedit_utils_get_window_workspace (window);
-
-	if (ws != workspace && ws != GEDIT_ALL_WORKSPACES)
-	{
-		return FALSE;
-	}
-
-	/* Check for viewport match */
-	gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
-	gdk_window_get_position (gdkwindow, &x, &y);
-	width = gdk_window_get_width (gdkwindow);
-	height = gdk_window_get_height (gdkwindow);
-
-	x += viewport_x;
-	y += viewport_y;
-
-	monitor = gdk_display_get_monitor_at_window(display, gdkwindow);
-	gdk_monitor_get_geometry(monitor, &geometry);
-
-	return x + width * .75 >= geometry.x &&
-	       x + width * .25 <= geometry.x + geometry.width &&
-	       y + height * .75 >= geometry.y &&
-	       y + height * .25 <= geometry.y + geometry.height;
-}
-
 static GeditWindow *
 get_active_window (GtkApplication *app)
 {
-	GdkScreen *screen;
-	guint workspace;
-	gint viewport_x, viewport_y;
-	GList *windows, *l;
+	GList *windows;
+	GList *l;
 
-	screen = gdk_screen_get_default ();
-
-	workspace = gedit_utils_get_current_workspace (screen);
-	gedit_utils_get_current_viewport (screen, &viewport_x, &viewport_y);
-
-	/* Gtk documentation says the window list is always in MRU order */
+	/* Gtk documentation says the window list is always in MRU order. */
 	windows = gtk_application_get_windows (app);
 	for (l = windows; l != NULL; l = l->next)
 	{
 		GtkWindow *window = l->data;
 
-		if (GEDIT_IS_WINDOW (window) && is_in_viewport (window, screen, workspace, viewport_x, viewport_y))
+		if (GEDIT_IS_WINDOW (window))
 		{
 			return GEDIT_WINDOW (window);
 		}
@@ -736,18 +637,52 @@ show_menubar (void)
 	return result;
 }
 
-static void
-setup_metadata_manager (void)
+static GFile *
+get_metadata_manager_file (void)
 {
-	const gchar *user_data_dir;
-	gchar *metadata_path;
+	return g_file_new_build_filename (gedit_dirs_get_user_data_dir (),
+					  "gedit-metadata.xml",
+					  NULL);
+}
 
-	user_data_dir = gedit_dirs_get_user_data_dir ();
-	metadata_path = g_build_filename (user_data_dir, "gedit-metadata.xml", NULL);
+static void
+load_metadata_manager (void)
+{
+	TeplMetadataManager *manager;
+	GFile *file;
+	GError *error = NULL;
 
-	tepl_metadata_manager_init (metadata_path);
+	manager = tepl_metadata_manager_get_singleton ();
+	file = get_metadata_manager_file ();
+	tepl_metadata_manager_load_from_disk (manager, file, &error);
 
-	g_free (metadata_path);
+	if (error != NULL)
+	{
+		g_warning ("Failed to load metadata: %s", error->message);
+		g_clear_error (&error);
+	}
+
+	g_object_unref (file);
+}
+
+static void
+save_metadata_manager (void)
+{
+	TeplMetadataManager *manager;
+	GFile *file;
+	GError *error = NULL;
+
+	manager = tepl_metadata_manager_get_singleton ();
+	file = get_metadata_manager_file ();
+	tepl_metadata_manager_save_to_disk (manager, file, TRUE, &error);
+
+	if (error != NULL)
+	{
+		g_warning ("Failed to save metadata: %s", error->message);
+		g_clear_error (&error);
+	}
+
+	g_object_unref (file);
 }
 
 static void
@@ -765,18 +700,15 @@ gedit_app_startup (GApplication *application)
 	gedit_debug_init ();
 	gedit_debug_message (DEBUG_APP, "Startup");
 
-	setup_metadata_manager ();
+	load_metadata_manager ();
 
 	setup_theme_extensions (GEDIT_APP (application));
 
-	/* Load settings */
-	priv->settings = gedit_settings_new ();
+	/* Load/init settings */
+	_gedit_settings_get_singleton ();
 	priv->ui_settings = g_settings_new ("org.gnome.gedit.preferences.ui");
 	priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
 
-	/* initial lockdown state */
-	priv->lockdown = gedit_settings_get_lockdown (priv->settings);
-
 	g_action_map_add_action_entries (G_ACTION_MAP (application),
 	                                 app_entries,
 	                                 G_N_ELEMENTS (app_entries),
@@ -798,6 +730,7 @@ gedit_app_startup (GApplication *application)
 	add_accelerator (GTK_APPLICATION (application), "app.new-window", "<Primary>N");
 	add_accelerator (GTK_APPLICATION (application), "app.quit", "<Primary>Q");
 	add_accelerator (GTK_APPLICATION (application), "app.help", "F1");
+	add_accelerator (GTK_APPLICATION (application), "app.shortcuts", "<Primary>question");
 
 	add_accelerator (GTK_APPLICATION (application), "win.hamburger-menu", "F10");
 	add_accelerator (GTK_APPLICATION (application), "win.open", "<Primary>O");
@@ -1230,6 +1163,7 @@ gedit_app_shutdown (GApplication *app)
 	save_accels ();
 	save_page_setup (GEDIT_APP (app));
 	save_print_settings (GEDIT_APP (app));
+	save_metadata_manager ();
 
 	G_APPLICATION_CLASS (gedit_app_parent_class)->shutdown (app);
 
@@ -1281,7 +1215,6 @@ gedit_app_class_init (GeditAppClass *klass)
 	GApplicationClass *app_class = G_APPLICATION_CLASS (klass);
 
 	object_class->dispose = gedit_app_dispose;
-	object_class->get_property = gedit_app_get_property;
 
 	app_class->startup = gedit_app_startup;
 	app_class->activate = gedit_app_activate;
@@ -1294,16 +1227,6 @@ gedit_app_class_init (GeditAppClass *klass)
 	klass->help_link_id = gedit_app_help_link_id_impl;
 	klass->set_window_title = gedit_app_set_window_title_impl;
 	klass->create_window = gedit_app_create_window_impl;
-
-	properties[PROP_LOCKDOWN] =
-		g_param_spec_flags ("lockdown",
-		                    "Lockdown",
-		                    "The lockdown mask",
-		                    GEDIT_TYPE_LOCKDOWN_MASK,
-		                    0,
-		                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
-
-	g_object_class_install_properties (object_class, LAST_PROP, properties);
 }
 
 static void
@@ -1387,10 +1310,6 @@ gedit_app_init (GeditApp *app)
 	gtk_window_set_default_icon_name ("org.gnome.gedit");
 
 	g_application_add_main_option_entries (G_APPLICATION (app), options);
-
-#ifdef ENABLE_INTROSPECTION
-	g_application_add_option_group (G_APPLICATION (app), g_irepository_get_option_group ());
-#endif
 }
 
 /**
@@ -1543,26 +1462,6 @@ gedit_app_get_views (GeditApp *app)
 	return res;
 }
 
-/**
- * gedit_app_get_lockdown:
- * @app: a #GeditApp
- *
- * Gets the lockdown mask (see #GeditLockdownMask) for the application.
- * The lockdown mask determines which functions are locked down using
- * the GNOME-wise lockdown GConf keys.
- **/
-GeditLockdownMask
-gedit_app_get_lockdown (GeditApp *app)
-{
-	GeditAppPrivate *priv;
-
-	g_return_val_if_fail (GEDIT_IS_APP (app), GEDIT_LOCKDOWN_ALL);
-
-	priv = gedit_app_get_instance_private (app);
-
-	return priv->lockdown;
-}
-
 gboolean
 gedit_app_show_help (GeditApp    *app,
                      GtkWindow   *parent,
@@ -1651,64 +1550,6 @@ find_extension_point_section (GMenuModel  *model,
 	return section;
 }
 
-static void
-app_lockdown_changed (GeditApp *app)
-{
-	GeditAppPrivate *priv;
-	GList *windows, *l;
-
-	priv = gedit_app_get_instance_private (app);
-
-	windows = gtk_application_get_windows (GTK_APPLICATION (app));
-	for (l = windows; l != NULL; l = g_list_next (l))
-	{
-		if (GEDIT_IS_WINDOW (l->data))
-		{
-			_gedit_window_set_lockdown (GEDIT_WINDOW (l->data),
-			                            priv->lockdown);
-		}
-	}
-
-	g_object_notify (G_OBJECT (app), "lockdown");
-}
-
-void
-_gedit_app_set_lockdown (GeditApp          *app,
-			 GeditLockdownMask  lockdown)
-{
-	GeditAppPrivate *priv;
-
-	g_return_if_fail (GEDIT_IS_APP (app));
-
-	priv = gedit_app_get_instance_private (app);
-
-	priv->lockdown = lockdown;
-	app_lockdown_changed (app);
-}
-
-void
-_gedit_app_set_lockdown_bit (GeditApp          *app,
-			     GeditLockdownMask  bit,
-			     gboolean           value)
-{
-	GeditAppPrivate *priv;
-
-	g_return_if_fail (GEDIT_IS_APP (app));
-
-	priv = gedit_app_get_instance_private (app);
-
-	if (value)
-	{
-		priv->lockdown |= bit;
-	}
-	else
-	{
-		priv->lockdown &= ~bit;
-	}
-
-	app_lockdown_changed (app);
-}
-
 /* Returns a copy */
 GtkPageSetup *
 _gedit_app_get_default_page_setup (GeditApp *app)
@@ -1778,18 +1619,6 @@ _gedit_app_set_default_print_settings (GeditApp         *app,
 	priv->print_settings = g_object_ref (settings);
 }
 
-GeditSettings *
-_gedit_app_get_settings (GeditApp *app)
-{
-	GeditAppPrivate *priv;
-
-	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
-
-	priv = gedit_app_get_instance_private (app);
-
-	return priv->settings;
-}
-
 GMenuModel *
 _gedit_app_get_hamburger_menu (GeditApp *app)
 {
