@@ -47,8 +47,7 @@ typedef struct
 {
 	GtkSourceFile *file;
 
-	/* Used only internally for the TeplFileMetadata. */
-	TeplFile *tepl_file;
+	TeplMetadata *metadata;
 
 	GSettings   *editor_settings;
 
@@ -107,6 +106,40 @@ static GHashTable *allocated_untitled_numbers = NULL;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GeditDocument, gedit_document, GTK_SOURCE_TYPE_BUFFER)
 
+static void
+load_metadata_from_metadata_manager (GeditDocument *doc)
+{
+	GeditDocumentPrivate *priv = gedit_document_get_instance_private (doc);
+	GFile *location;
+
+	location = gtk_source_file_get_location (priv->file);
+
+	if (location != NULL)
+	{
+		TeplMetadataManager *manager;
+
+		manager = tepl_metadata_manager_get_singleton ();
+		tepl_metadata_manager_copy_from (manager, location, priv->metadata);
+	}
+}
+
+static void
+save_metadata_into_metadata_manager (GeditDocument *doc)
+{
+	GeditDocumentPrivate *priv = gedit_document_get_instance_private (doc);
+	GFile *location;
+
+	location = gtk_source_file_get_location (priv->file);
+
+	if (location != NULL)
+	{
+		TeplMetadataManager *manager;
+
+		manager = tepl_metadata_manager_get_singleton ();
+		tepl_metadata_manager_merge_into (manager, location, priv->metadata);
+	}
+}
+
 static gint
 get_untitled_number (void)
 {
@@ -209,12 +242,12 @@ gedit_document_dispose (GObject *object)
 	/* Metadata must be saved here and not in finalize because the language
 	 * is gone by the time finalize runs.
 	 */
-	if (priv->tepl_file != NULL)
+	if (priv->metadata != NULL)
 	{
 		save_metadata (doc);
 
-		g_object_unref (priv->tepl_file);
-		priv->tepl_file = NULL;
+		g_object_unref (priv->metadata);
+		priv->metadata = NULL;
 	}
 
 	g_clear_object (&priv->file);
@@ -604,32 +637,75 @@ save_encoding_metadata (GeditDocument *doc)
 				     NULL);
 }
 
-static GtkSourceStyleScheme *
-get_default_style_scheme (GSettings *editor_settings)
+static gchar *
+get_default_style_scheme_id (void)
 {
+	GeditSettings *settings;
+	GSettings *editor_settings;
+	GVariant *default_value;
+	gchar *default_style_scheme_id;
+
+	settings = _gedit_settings_get_singleton ();
+	editor_settings = _gedit_settings_peek_editor_settings (settings);
+
+	default_value = g_settings_get_default_value (editor_settings, GEDIT_SETTINGS_SCHEME);
+	default_style_scheme_id = g_variant_dup_string (default_value, NULL);
+	g_variant_unref (default_value);
+
+	return default_style_scheme_id;
+}
+
+static void
+update_style_scheme (GeditDocument *doc)
+{
+	GeditSettings *settings;
+	GSettings *editor_settings;
+	gchar *style_scheme_id;
 	GtkSourceStyleSchemeManager *manager;
-	gchar *scheme_id;
-	GtkSourceStyleScheme *def_style;
+	GtkSourceStyleScheme *style_scheme = NULL;
+
+	settings = _gedit_settings_get_singleton ();
+	editor_settings = _gedit_settings_peek_editor_settings (settings);
+	style_scheme_id = g_settings_get_string (editor_settings, GEDIT_SETTINGS_SCHEME);
 
 	manager = gtk_source_style_scheme_manager_get_default ();
-	scheme_id = g_settings_get_string (editor_settings, GEDIT_SETTINGS_SCHEME);
-	def_style = gtk_source_style_scheme_manager_get_scheme (manager, scheme_id);
 
-	if (def_style == NULL)
+	if (style_scheme_id != NULL)
 	{
-		g_warning ("Default style scheme '%s' cannot be found, falling back to 'classic' style scheme ", scheme_id);
-
-		def_style = gtk_source_style_scheme_manager_get_scheme (manager, "classic");
-
-		if (def_style == NULL)
-		{
-			g_warning ("Style scheme 'classic' cannot be found, check your GtkSourceView installation.");
-		}
+		style_scheme = gtk_source_style_scheme_manager_get_scheme (manager, style_scheme_id);
 	}
 
-	g_free (scheme_id);
+	if (style_scheme == NULL)
+	{
+		gchar *default_style_scheme_id;
 
-	return def_style;
+		default_style_scheme_id = get_default_style_scheme_id ();
+
+		g_warning_once ("Style scheme '%s' cannot be found, falling back to '%s' default style scheme.",
+				style_scheme_id,
+				default_style_scheme_id);
+
+		style_scheme = gtk_source_style_scheme_manager_get_scheme (manager, default_style_scheme_id);
+		if (style_scheme == NULL)
+		{
+			g_warning_once ("Default style scheme '%s' cannot be found, check your GtkSourceView installation.",
+					default_style_scheme_id);
+		}
+
+		g_free (default_style_scheme_id);
+	}
+
+	gtk_source_buffer_set_style_scheme (GTK_SOURCE_BUFFER (doc), style_scheme);
+
+	g_free (style_scheme_id);
+}
+
+static void
+editor_settings_scheme_changed_cb (GSettings     *editor_settings,
+				   const gchar   *key,
+				   GeditDocument *doc)
+{
+	update_style_scheme (doc);
 }
 
 static GtkSourceLanguage *
@@ -716,6 +792,8 @@ on_location_changed (GtkSourceFile *file,
 
 	priv = gedit_document_get_instance_private (doc);
 
+	load_metadata_from_metadata_manager (doc);
+
 	location = gtk_source_file_get_location (file);
 
 	if (location != NULL && priv->untitled_number > 0)
@@ -727,47 +805,12 @@ on_location_changed (GtkSourceFile *file,
 	g_object_notify_by_pspec (G_OBJECT (doc), properties[PROP_SHORTNAME]);
 }
 
-static void
-on_tepl_location_changed (TeplFile      *file,
-			  GParamSpec    *pspec,
-			  GeditDocument *doc)
-{
-	TeplFileMetadata *metadata;
-	GError *error = NULL;
-
-	/* Load metadata for this location: we load sync since metadata is
-	 * always local so it should be fast and we need the information
-	 * right after the location was set.
-	 * TODO: do async I/O for the metadata.
-	 */
-	metadata = tepl_file_get_file_metadata (file);
-	tepl_file_metadata_load (metadata, NULL, &error);
-
-	if (error != NULL)
-	{
-		/* Do not complain about metadata if we are opening a
-		 * non existing file.
-		 * TODO: we should know beforehand whether the file exists or
-		 * not, and load the metadata only when needed (after loading
-		 * the file content, for example).
-		 */
-		if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_ISDIR) &&
-		    !g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOTDIR) &&
-		    !g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT) &&
-		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-		{
-			g_warning ("Loading metadata failed: %s", error->message);
-		}
-
-		g_clear_error (&error);
-	}
-}
-
 static void
 gedit_document_init (GeditDocument *doc)
 {
+	GeditSettings *settings;
+	GSettings *editor_settings;
 	GeditDocumentPrivate *priv;
-	GtkSourceStyleScheme *style_scheme;
 
 	gedit_debug (DEBUG_DOCUMENT);
 
@@ -782,6 +825,7 @@ gedit_document_init (GeditDocument *doc)
 	update_time_of_last_save_or_load (doc);
 
 	priv->file = gtk_source_file_new ();
+	priv->metadata = tepl_metadata_new ();
 
 	g_signal_connect_object (priv->file,
 				 "notify::location",
@@ -789,19 +833,6 @@ gedit_document_init (GeditDocument *doc)
 				 doc,
 				 0);
 
-	priv->tepl_file = tepl_file_new ();
-
-	g_signal_connect_object (priv->tepl_file,
-				 "notify::location",
-				 G_CALLBACK (on_tepl_location_changed),
-				 doc,
-				 0);
-
-	/* For using TeplFileMetadata we only need the TeplFile:location. */
-	g_object_bind_property (priv->file, "location",
-				priv->tepl_file, "location",
-				G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
-
 	g_settings_bind (priv->editor_settings,
 	                 GEDIT_SETTINGS_MAX_UNDO_ACTIONS,
 	                 doc,
@@ -820,11 +851,15 @@ gedit_document_init (GeditDocument *doc)
 	                 "highlight-matching-brackets",
 	                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_NO_SENSITIVITY);
 
-	style_scheme = get_default_style_scheme (priv->editor_settings);
-	if (style_scheme != NULL)
-	{
-		gtk_source_buffer_set_style_scheme (GTK_SOURCE_BUFFER (doc), style_scheme);
-	}
+	settings = _gedit_settings_get_singleton ();
+	editor_settings = _gedit_settings_peek_editor_settings (settings);
+	g_signal_connect_object (editor_settings,
+				 "changed::" GEDIT_SETTINGS_SCHEME,
+				 G_CALLBACK (editor_settings_scheme_changed_cb),
+				 doc,
+				 0);
+
+	update_style_scheme (doc);
 
 	g_signal_connect (doc,
 			  "notify::content-type",
@@ -1243,8 +1278,14 @@ _gedit_document_needs_saving (GeditDocument *doc)
 	return (externally_modified || deleted) && !priv->create;
 }
 
-/* If @line is bigger than the lines of the document, the cursor is moved
- * to the last line and FALSE is returned.
+/**
+ * gedit_document_goto_line:
+ * @doc: a #GeditDocument.
+ * @line: the line number.
+ *
+ * Returns: if @line is bigger than the lines of the document, the cursor is moved
+ * to the last line and %FALSE is returned.
+ * Deprecated: 3.38: Use tepl_view_goto_line() instead.
  */
 gboolean
 gedit_document_goto_line (GeditDocument *doc,
@@ -1266,6 +1307,15 @@ gedit_document_goto_line (GeditDocument *doc,
 	return gtk_text_iter_get_line (&iter) == line;
 }
 
+/**
+ * gedit_document_goto_line_offset:
+ * @doc: a #GeditDocument.
+ * @line: the line.
+ * @line_offset: the line offset.
+ *
+ * Returns: the return value.
+ * Deprecated: 3.38: Use tepl_view_goto_line_offset() instead.
+ */
 gboolean
 gedit_document_goto_line_offset (GeditDocument *doc,
 				 gint           line,
@@ -1360,20 +1410,18 @@ gedit_document_get_metadata (GeditDocument *doc,
 			     const gchar   *key)
 {
 	GeditDocumentPrivate *priv;
-	TeplFileMetadata *metadata;
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 
 	priv = gedit_document_get_instance_private (doc);
 
-	if (priv->tepl_file == NULL)
+	if (priv->metadata == NULL)
 	{
 		return NULL;
 	}
 
-	metadata = tepl_file_get_file_metadata (priv->tepl_file);
-	return tepl_file_metadata_get (metadata, key);
+	return tepl_metadata_get (priv->metadata, key);
 }
 
 /**
@@ -1391,57 +1439,30 @@ gedit_document_set_metadata (GeditDocument *doc,
 			     ...)
 {
 	GeditDocumentPrivate *priv;
-	TeplFileMetadata *metadata;
 	va_list var_args;
 	const gchar *key;
-	GError *error = NULL;
 
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (first_key != NULL);
 
 	priv = gedit_document_get_instance_private (doc);
 
-	if (priv->tepl_file == NULL)
+	if (priv->metadata == NULL)
 	{
 		return;
 	}
 
-	metadata = tepl_file_get_file_metadata (priv->tepl_file);
 	va_start (var_args, first_key);
 
 	for (key = first_key; key != NULL; key = va_arg (var_args, const gchar *))
 	{
 		const gchar *value = va_arg (var_args, const gchar *);
-		tepl_file_metadata_set (metadata, key, value);
+		tepl_metadata_set (priv->metadata, key, value);
 	}
 
 	va_end (var_args);
 
-	/* We save synchronously since metadata is always local so it should be
-	 * fast. Moreover this function can be called on application shutdown,
-	 * when the main loop has already exited, so an async operation would
-	 * not terminate.
-	 * https://bugzilla.gnome.org/show_bug.cgi?id=736591
-	 * TODO: do async I/O for the metadata.
-	 */
-	tepl_file_metadata_save (metadata, NULL, &error);
-
-	if (error != NULL)
-	{
-		/* Do not complain about metadata if we are closing a document
-		 * for a non existing file.
-		 * TODO: we should know beforehand whether the file exists or
-		 * not, and save the metadata only when needed (after saving the
-		 * file content, for example).
-		 */
-		if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT) &&
-		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-		{
-			g_warning ("Saving metadata failed: %s", error->message);
-		}
-
-		g_clear_error (&error);
-	}
+	save_metadata_into_metadata_manager (doc);
 }
 
 static void
