@@ -29,7 +29,7 @@
 
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
-#include <gtksourceview/gtksource.h>
+#include <tepl/tepl.h>
 #include <libpeas-gtk/peas-gtk.h>
 
 #include "gedit-utils.h"
@@ -39,8 +39,6 @@
 #include "gedit-settings.h"
 #include "gedit-utils.h"
 #include "gedit-file-chooser-dialog.h"
-#include "gedit-app.h"
-#include "gedit-app-private.h"
 
 /*
  * gedit-preferences dialog is a singleton since we don't
@@ -93,7 +91,6 @@ struct _GeditPreferencesDialog
 	GtkWidget	*schemes_list;
 	GtkWidget	*install_scheme_button;
 	GtkWidget	*uninstall_scheme_button;
-	GtkWidget	*schemes_scrolled_window;
 	GtkWidget	*schemes_toolbar;
 
 	GeditFileChooserDialog *
@@ -199,7 +196,6 @@ gedit_preferences_dialog_class_init (GeditPreferencesDialogClass *klass)
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, font_button);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, font_grid);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, schemes_list);
-	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, schemes_scrolled_window);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, install_scheme_button);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, uninstall_scheme_button);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, schemes_toolbar);
@@ -446,7 +442,7 @@ setup_font_colors_page_font_section (GeditPreferencesDialog *dlg)
 			 _("Click on this button to select the font to be used by the editor"));
 
 	/* Get values */
-	settings = _gedit_app_get_settings (GEDIT_APP (g_application_get_default ()));
+	settings = _gedit_settings_get_singleton ();
 	system_font = gedit_settings_get_system_font (settings);
 
 	label = g_strdup_printf(_("_Use the system fixed width font (%s)"),
@@ -475,65 +471,33 @@ setup_font_colors_page_font_section (GeditPreferencesDialog *dlg)
 }
 
 static void
-set_buttons_sensisitivity_according_to_scheme (GeditPreferencesDialog *dlg,
-                                               GtkSourceStyleScheme   *scheme)
+update_style_scheme_buttons_sensisitivity (GeditPreferencesDialog *dlg)
 {
+	GtkSourceStyleScheme *selected_style_scheme;
 	gboolean editable = FALSE;
 
-	if (scheme != NULL)
+	selected_style_scheme = gtk_source_style_scheme_chooser_get_style_scheme (GTK_SOURCE_STYLE_SCHEME_CHOOSER (dlg->schemes_list));
+
+	if (selected_style_scheme != NULL)
 	{
 		const gchar *filename;
 
-		filename = gtk_source_style_scheme_get_filename (scheme);
+		filename = gtk_source_style_scheme_get_filename (selected_style_scheme);
 		if (filename != NULL)
 		{
 			editable = g_str_has_prefix (filename, gedit_dirs_get_user_styles_dir ());
 		}
 	}
 
-	gtk_widget_set_sensitive (dlg->uninstall_scheme_button,
-	                          editable);
+	gtk_widget_set_sensitive (dlg->uninstall_scheme_button, editable);
 }
 
 static void
-style_scheme_changed (GtkSourceStyleSchemeChooser *chooser,
-                      GParamSpec                  *pspec,
-                      GeditPreferencesDialog      *dlg)
+style_scheme_notify_cb (GtkSourceStyleSchemeChooser *chooser,
+			GParamSpec                  *pspec,
+			GeditPreferencesDialog      *dlg)
 {
-	GtkSourceStyleScheme *scheme;
-	const gchar *id;
-
-	scheme = gtk_source_style_scheme_chooser_get_style_scheme (chooser);
-	id = gtk_source_style_scheme_get_id (scheme);
-
-	g_settings_set_string (dlg->editor, GEDIT_SETTINGS_SCHEME, id);
-	set_buttons_sensisitivity_according_to_scheme (dlg, scheme);
-}
-
-static GtkSourceStyleScheme *
-get_default_color_scheme (GeditPreferencesDialog *dlg)
-{
-	GtkSourceStyleSchemeManager *manager;
-	GtkSourceStyleScheme *scheme = NULL;
-	gchar *pref_id;
-
-	manager = gtk_source_style_scheme_manager_get_default ();
-
-	pref_id = g_settings_get_string (dlg->editor,
-	                                 GEDIT_SETTINGS_SCHEME);
-
-	scheme = gtk_source_style_scheme_manager_get_scheme (manager,
-	                                                     pref_id);
-	g_free (pref_id);
-
-	if (scheme == NULL)
-	{
-		/* Fall-back to classic style scheme */
-		scheme = gtk_source_style_scheme_manager_get_scheme (manager,
-		                                                     "classic");
-	}
-
-	return scheme;
+	update_style_scheme_buttons_sensisitivity (dlg);
 }
 
 /*
@@ -617,7 +581,6 @@ file_copy (const gchar  *name,
 
 /*
  * install_style_scheme:
- * @manager: a #GtkSourceStyleSchemeManager
  * @fname: the file name of the style scheme to be installed
  *
  * Install a new user scheme.
@@ -713,17 +676,13 @@ install_style_scheme (const gchar *fname)
 	return NULL;
 }
 
-/**
+/*
  * uninstall_style_scheme:
- * @manager: a #GtkSourceStyleSchemeManager
  * @scheme: a #GtkSourceStyleScheme
  *
  * Uninstall a user scheme.
  *
- * If the call was succesful, it returns %TRUE
- * otherwise %FALSE.
- *
- * Return value: %TRUE on success, %FALSE otherwise.
+ * Returns: %TRUE on success, %FALSE otherwise.
  */
 static gboolean
 uninstall_style_scheme (GtkSourceStyleScheme *scheme)
@@ -785,16 +744,14 @@ add_scheme_chooser_response_cb (GeditFileChooserDialog *chooser,
 
 	if (scheme == NULL)
 	{
-		gedit_warning (GTK_WINDOW (dlg),
-		               _("The selected color scheme cannot be installed."));
+		tepl_utils_show_warning_dialog (GTK_WINDOW (dlg),
+						_("The selected color scheme cannot be installed."));
 
 		return;
 	}
 
 	g_settings_set_string (dlg->editor, GEDIT_SETTINGS_SCHEME,
 	                       gtk_source_style_scheme_get_id (scheme));
-
-	set_buttons_sensisitivity_according_to_scheme (dlg, scheme);
 }
 
 static void
@@ -843,14 +800,33 @@ uninstall_scheme_clicked (GtkButton              *button,
 			  GeditPreferencesDialog *dlg)
 {
 	GtkSourceStyleScheme *scheme;
+	GtkSourceStyleScheme *new_selected_scheme;
 
 	scheme = gtk_source_style_scheme_chooser_get_style_scheme (GTK_SOURCE_STYLE_SCHEME_CHOOSER (dlg->schemes_list));
 
+	if (scheme == NULL)
+	{
+		return;
+	}
+
 	if (!uninstall_style_scheme (scheme))
 	{
-		gedit_warning (GTK_WINDOW (dlg),
-		               _("Could not remove color scheme “%s”."),
-		               gtk_source_style_scheme_get_name (scheme));
+		tepl_utils_show_warning_dialog (GTK_WINDOW (dlg),
+						_("Could not remove color scheme “%s”."),
+						gtk_source_style_scheme_get_name (scheme));
+		return;
+	}
+
+	new_selected_scheme = gtk_source_style_scheme_chooser_get_style_scheme (GTK_SOURCE_STYLE_SCHEME_CHOOSER (dlg->schemes_list));
+	if (new_selected_scheme == NULL)
+	{
+		GeditSettings *settings;
+		GSettings *editor_settings;
+
+		settings = _gedit_settings_get_singleton ();
+		editor_settings = _gedit_settings_peek_editor_settings (settings);
+
+		g_settings_reset (editor_settings, GEDIT_SETTINGS_SCHEME);
 	}
 }
 
@@ -858,14 +834,13 @@ static void
 setup_font_colors_page_style_scheme_section (GeditPreferencesDialog *dlg)
 {
 	GtkStyleContext *context;
-	GtkSourceStyleScheme *scheme;
+	GeditSettings *settings;
+	GSettings *editor_settings;
 
 	gedit_debug (DEBUG_PREFS);
 
-	scheme = get_default_color_scheme (dlg);
-
-	/* junction between the scrolled window and the toolbar */
-	context = gtk_widget_get_style_context (dlg->schemes_scrolled_window);
+	/* junction between the schemes list and the toolbar */
+	context = gtk_widget_get_style_context (dlg->schemes_list);
 	gtk_style_context_set_junction_sides (context, GTK_JUNCTION_BOTTOM);
 	context = gtk_widget_get_style_context (dlg->schemes_toolbar);
 	gtk_style_context_set_junction_sides (context, GTK_JUNCTION_TOP);
@@ -873,7 +848,7 @@ setup_font_colors_page_style_scheme_section (GeditPreferencesDialog *dlg)
 	/* Connect signals */
 	g_signal_connect (dlg->schemes_list,
 	                  "notify::style-scheme",
-	                  G_CALLBACK (style_scheme_changed),
+	                  G_CALLBACK (style_scheme_notify_cb),
 	                  dlg);
 	g_signal_connect (dlg->install_scheme_button,
 			  "clicked",
@@ -884,11 +859,13 @@ setup_font_colors_page_style_scheme_section (GeditPreferencesDialog *dlg)
 			  G_CALLBACK (uninstall_scheme_clicked),
 			  dlg);
 
-	gtk_source_style_scheme_chooser_set_style_scheme (GTK_SOURCE_STYLE_SCHEME_CHOOSER (dlg->schemes_list),
-	                                                  scheme);
+	settings = _gedit_settings_get_singleton ();
+	editor_settings = _gedit_settings_peek_editor_settings (settings);
+	g_settings_bind (editor_settings, GEDIT_SETTINGS_SCHEME,
+			 dlg->schemes_list, "tepl-style-scheme-id",
+			 G_SETTINGS_BIND_DEFAULT);
 
-	/* Set initial widget sensitivity */
-	set_buttons_sensisitivity_according_to_scheme (dlg, scheme);
+	update_style_scheme_buttons_sensisitivity (dlg);
 }
 
 static void
