@@ -26,8 +26,6 @@
 #  include "gedit-app-osx.h"
 #elif defined G_OS_WIN32
 #  include "gedit-app-win32.h"
-#else
-#  include "gedit-app-x11.h"
 #endif
 
 #include <locale.h>
@@ -36,6 +34,7 @@
 
 #include "gedit-dirs.h"
 #include "gedit-debug.h"
+#include "gedit-settings.h"
 
 #ifdef G_OS_WIN32
 #include <gmodule.h>
@@ -130,7 +129,7 @@ main (int argc, char *argv[])
 
 	type = GEDIT_TYPE_APP_WIN32;
 #else
-	type = GEDIT_TYPE_APP_X11;
+	type = GEDIT_TYPE_APP;
 #endif
 
 	/* NOTE: we should not make any calls to the gedit api before the
@@ -147,6 +146,8 @@ main (int argc, char *argv[])
 
 	status = g_application_run (G_APPLICATION (app), argc, argv);
 
+	gedit_settings_unref_singleton ();
+
 	/* Break reference cycles caused by the PeasExtensionSet
 	 * for GeditAppActivatable which holds a ref on the GeditApp
 	 */
